package Drag_n_Drop;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.datatransfer.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import javax.swing.text.*;
/**
 * Build the tree view; Start listenning TCP connection;
 * Create instance of Bservice to register the service; 
 * Response to the "drag and drop event" by creating send class to 
 * initiating sending process.
 * Implement Blistener interface to update the tree view. 
 * @see  OwnTree 
 */
public class FSTree extends JTree implements BListener{
        Send s;
        //service used to DNS register,resolve,update
        public BService service;
        boolean txtRecPub;
        String[] txtName;
        int txtNameSize;
        boolean rmvFlag;
        //Start listenning TCP connection
        public Receive ft=new Receive();
        //Stroe the <hostname,port> and <IPAddress,port> pair
        HashMap<String,String> nodes=new HashMap<String,String>();
        public FSTree() {
		this("");
	}
 	/**
         * Constructor
         * @param title the name of the tree root 
         */
        public FSTree (String title) {
                //build the tree
         	super(new OwnTree(title));
                txtRecPub = false;
                service=new BService("_7ds_filexchange._udp");
                service.register(null,FileXchange.service_port);
                // set this listerner to receive update
                service.setListener(this);
               init();
	}
	/**
         * Disable dragging action on the tree;
         * Set an event listener for drag and drop event
         */
	private void init() {
		// We don't want to export anything from this tree, only import
		setDragEnabled(false);
		setTransferHandler(new FSTransfer());
  	}
        /**
         * Resopond to new service found event.
         * Add the new node to the tree view, store the host information.
         * @param n    the node to be added.
         */
       public synchronized void  serviceUpdated(BNode n){// update  nodes
           String prefix=n.getHostName();
           if(nodes.containsKey(prefix+"("+n.getPort()+")"))
               return;
           OwnTree root = (OwnTree)getModel().getRoot();
           root.add(new OwnTree(prefix+"("+n.getPort()+")"));
          int k=root.getDepth();
          ((DefaultTreeModel)getModel()).reload();
           int startRow = 0;
           TreePath path = getNextMatch(prefix+"("+n.getPort()+")", startRow, Position.Bias.Forward);
           //int txtSize = n..txtRecord.size();
           OwnTree node =(OwnTree)path.getLastPathComponent();
           nodes.put(prefix+"("+n.getPort()+")",n.getHostName()+"/"+n.getPort());
    }
    /**
     * Respond to service lost event, delete corresponding node in the tree view and host information.
     * @param n   the node to be deleted.
     */
         public synchronized void serviceExited(BNode n){// nodes exit the network update
             
             String prefix=n.getHostName();
             if(!nodes.containsKey(prefix+"("+n.getPort()+")"))
                 return;
             TreePath path = getNextMatch(prefix+"("+n.getPort()+")",0,Position.Bias.Forward);
             OwnTree node = (OwnTree)path.getLastPathComponent();
             node.removeAllChildren();
             node.removeFromParent();
             ((DefaultTreeModel)getModel()).reload();
             nodes.remove(prefix+"("+n.getPort()+")");
              //expand the nodes
             int i=0;
             while( i < getRowCount() ) {
                expandRow( i );
                i++;
               }    
         }
         
     /**
      * Get the host's IP address and port number from the service name.
      * @param serviceName the host name in the hash map as key
      * @return the host ip address and port number or false if the node is not in the map
      */
      public String getHost(String serviceName){//search the table to get the IP address and port
             if(nodes.containsKey(serviceName)==true)
                return "/"+nodes.get(serviceName);    
             else
                return "false";
      } 
      /**
       * Inner class, handle the drag-and-drop action.
       * Create instance of Send to start sending data to the target node.
       * @see  Send
       */
      public class FSTransfer extends TransferHandler {
		/**
                 * Import data, override method
                 * @param comp  component that receives the importing data, provided to enable
                 *              sharing of TransferHandlers
                 * @param t     the data to import
                 * @return      true if the data can be trasfered, false if exceptions happen
                 */
                public boolean importData(JComponent comp, Transferable t) {
                        FSTree tree;
                        try {
                               tree = (FSTree) comp;
                        } 
                        finally {
                        }
                        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
                        TreePath path = tree.getSelectionPath();
                        try {
                                List data = (List)t.getTransferData(DataFlavor.javaFileListFlavor);
                                Iterator i = data.iterator();
                                while (i.hasNext()) {
                                        File f = (File)i.next();
                                        sendFile(path, f);
                                }
                                model.reload();
                                return true;
                        }
                        catch (UnsupportedFlavorException ufe) {
                                System.err.println("Ack! we should not be here.\nBad Flavor.");
                        }
                        catch (IOException ioe) {
                                System.out.println("Something failed during import:\n" + ioe);
                        }
                        return false;
                }
		
		/**
                 * Override method, allow transfer
                 * @param comp             component that receives the importing data, provided to enable
                 *                         sharing of TransferHandlers
                 * @param transferFlavors  the data format available
                 * @return                 true if the data can be transferred 
                 */
               public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
            		return true;
		}
		
		private void sendFile(TreePath path, File f) {// send file
                        //send only if the nodes is a host
                       if(FileXchange.send_lock==true){
                            JOptionPane.showMessageDialog(FileXchange.f,"Sending file, please wait until last file transfer completed.");
                            return;
                        }
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
			try {
				//get the host IP address and port from the hash map
                                String host=getHost(node.toString());
                                // if it is not in the hash map, return
                                if(host.equals("false"))
                                    return;
                                else
                                    host=host.substring(1,host.length());
                                if(node.toString().contains(".local.")){
                                    String temp=new String(node.toString().replace(".local.",""));
                                    if(FileXchange.local_host.equals(temp)){
                                        JOptionPane.showMessageDialog(FileXchange.f,"Please do not send file to yourself! ");
                                        return;
                                    }
                                }    
                                s=new Send(host,node.toString(),f.getAbsolutePath());
                                //start send thread
                                s.start(); 
                        } 
                        catch (Exception ex) {
			      ex.printStackTrace();
			}
                        //record the sending history
                       String filename_temp=f.getAbsolutePath();
                        if(filename_temp.contains("\\")){
                         filename_temp=new String(filename_temp.split("\\\\")[filename_temp.split("\\\\").length-1]);
                       }
			node.add(new DefaultMutableTreeNode(filename_temp));    	
		}
	}
}
/**
 * This class is extending DefaultMutableTreeNode, used to decide whether 
 * the node is a leaf or not
 */
   class OwnTree extends DefaultMutableTreeNode{
        //ownTree a=new ownTree();
        OwnTree(String s){
        super(s);
    }
    /**
     * If the node's parent is the root, it is not a leaf
     * @return: true if it is a leaf, flase if it is not.
     */
        public boolean isLeaf(){
            if(this.getParent()==this.getRoot())
                return false;
            else if(this.isRoot())
                return false;
            else
                return true;
        }

}